#!/usr/bin/env python

# Copyright (c) CTU -- All Rights Reserved
# Created on: 2022-10-11
#     Author: Vladimir Petrik <vladimir.petrik@cvut.cz>
#

from setuptools import setup

if __name__ == '__main__':
    setup()
